/* Reading a frequency table
 *
 * 15-122 Principles of Imperative Computation, Fall 2012
 */

/* Function to read in a frequency table from a file  */
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <assert.h>
#include <stdbool.h>

#include "lib/contracts.h"
#include "lib/xalloc.h"
#include "freqtable.h"

/* Implementation */

bool all_positive(int *table)
//@requires \length(table) == 128;
{
  for (int i = 0; i < 128; i++)
    if (!(table[i] >= 0)) return false;
  return true;
}

bool is_freqtable(int *table)
//@requires \length(table) == 128;
{
  return table != NULL
    && table[0] == 0		/* '\0' not permitted */
    && all_positive(table);
}

freqtable read_freqtable(char *filename) {
  REQUIRES(filename != NULL);
  FILE* stream = fopen(filename, "r");
  if (stream == NULL) {
    perror(filename);
    exit(1);
  }
  freqtable table = xcalloc(128, sizeof(int));
  while (!feof(stream)) {
    char c;
    int n;
    int ret_code = fscanf(stream, "%1c:%d\n", &c, &n);
    if (ret_code != 2) {
      fprintf(stderr, "%s\n", "Could not parse line");
      exit(1);
    }
    table[(int)c] = n;
  }
  fclose(stream);
  ENSURES(is_freqtable(table));
  return table;
}

void freqtable_free(freqtable table) {
  free(table);
}
